'use strict';

exports.__esModule = true;
exports.getLoadableState = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* eslint-disable react/no-danger */


var getLoadableState = exports.getLoadableState = function () {
  var _ref2 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(rootElement) {
    var _this = this;

    var rootContext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var fetchRoot = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    var queries, errors, componentIds, mappedQueries, error;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            queries = getQueriesFromTree({ rootElement: rootElement, rootContext: rootContext }, fetchRoot);

            // no queries found, nothing to do

            if (queries.length) {
              _context2.next = 3;
              break;
            }

            return _context2.abrupt('return', new _DeferredState2.default([]));

          case 3:
            errors = [];
            componentIds = [];

            // wait on each query that we found, re-rendering the subtree when it's done

            mappedQueries = queries.map(function () {
              var _ref4 = _asyncToGenerator(regeneratorRuntime.mark(function _callee(_ref3) {
                var query = _ref3.query,
                    element = _ref3.element,
                    context = _ref3.context;
                var componentId, state;
                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _context.prev = 0;
                        _context.next = 3;
                        return query;

                      case 3:
                        componentId = _context.sent;

                        componentIds.push(componentId);
                        _context.next = 7;
                        return getLoadableState(element, context, false);

                      case 7:
                        state = _context.sent;

                        componentIds = [].concat(componentIds, state.componentIds);
                        _context.next = 14;
                        break;

                      case 11:
                        _context.prev = 11;
                        _context.t0 = _context['catch'](0);

                        errors.push(_context.t0);

                      case 14:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, _this, [[0, 11]]);
              }));

              return function (_x5) {
                return _ref4.apply(this, arguments);
              };
            }());
            _context2.next = 8;
            return Promise.all(mappedQueries);

          case 8:
            if (!(errors.length > 0)) {
              _context2.next = 12;
              break;
            }

            error = errors.length === 1 ? errors[0] : new Error(errors.length + ' errors were thrown when importing your modules.');

            error.queryErrors = errors;
            throw error;

          case 12:
            return _context2.abrupt('return', new _DeferredState2.default(componentIds));

          case 13:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function getLoadableState(_x4) {
    return _ref2.apply(this, arguments);
  };
}();

exports.walkTree = walkTree;

var _react = require('react');

var _constants = require('../constants');

var _DeferredState = require('./DeferredState');

var _DeferredState2 = _interopRequireDefault(_DeferredState);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// Recurse a React Element tree, running visitor on each element.
// If visitor returns `false`, don't call the element's render function
// or recurse into its child elements
function walkTree(element, context, visitor) {
  var Component = element.type;
  // a stateless functional component or a class
  if (typeof Component === 'function') {
    var props = _extends({}, Component.defaultProps, element.props);
    var childContext = context;
    var child = void 0;

    // Are we are a react class?
    //   https://github.com/facebook/react/blob/master/src/renderers/shared/stack/reconciler/ReactCompositeComponent.js#L66
    if (Component.prototype && Component.prototype.isReactComponent) {
      // typescript force casting since typescript doesn't have definitions for class
      // methods
      var instance = new Component(props, context);
      // In case the user doesn't pass these to super in the constructor
      instance.props = instance.props || props;
      instance.context = instance.context || context;

      // Override setState to just change the state, not queue up an update.
      //   (we can't do the default React thing as we aren't mounted "properly"
      //   however, we don't need to re-render as well only support setState in
      //   componentWillMount, which happens *before* render).
      instance.setState = function (nextState) {
        instance.state = _extends({}, instance.state, nextState);
      };

      // this is a poor man's version of
      //   https://github.com/facebook/react/blob/master/src/renderers/shared/stack/reconciler/ReactCompositeComponent.js#L181
      if (instance.componentWillMount) {
        instance.componentWillMount();
      }

      if (instance.getChildContext) {
        childContext = _extends({}, context, instance.getChildContext());
      }

      if (visitor(element, instance, context) === false) {
        return;
      }

      child = instance.render();
    } else {
      // just a stateless functional
      if (visitor(element, null, context) === false) {
        return;
      }

      // typescript casting for stateless component
      child = Component(props, context);
    }

    if (child) {
      walkTree(child, childContext, visitor);
    }
  } else {
    // a basic string or dom element, just get children
    if (visitor(element, null, context) === false) {
      return;
    }

    if (element.props && element.props.children) {
      _react.Children.forEach(element.props.children, function (child) {
        if (child) {
          walkTree(child, context, visitor);
        }
      });
    }
  }
}

function getQueriesFromTree(_ref) {
  var rootElement = _ref.rootElement,
      _ref$rootContext = _ref.rootContext,
      rootContext = _ref$rootContext === undefined ? {} : _ref$rootContext;
  var fetchRoot = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  var queries = [];

  walkTree(rootElement, rootContext, function (element, instance, context) {
    var skipRoot = !fetchRoot && element === rootElement;

    if (instance && instance.constructor[_constants.LOADABLE] && !skipRoot) {
      var loadable = instance.constructor[_constants.LOADABLE]();
      var query = loadable.load().then(function () {
        return loadable.componentId;
      });

      if (query) {
        queries.push({ query: query, element: element, context: context });

        // Tell walkTree to not recurse inside this component;  we will
        // wait for the query to execute before attempting it.
        return false;
      }
    }

    return true;
  });

  return queries;
}